﻿using DotNetNuke.Security;
using DotNetNuke.Web.Api;
using System.Web.Http;
using System.Collections.Generic;
using ToSic.SexyContent.WebApi;
using System.Linq;
using System;

public class FeedbackController : SxcApiController
{
    [HttpGet]
    [DnnModuleAuthorize(AccessLevel = SecurityAccessLevel.Anonymous)]
    [ValidateAntiForgeryToken]
    public dynamic Categories()
    {
        // The following line would just return all categories
        // We'll comment it, because we want to sort it first, which we do with the Query we prepared
        // return Sxc.Serializer.Prepare(App.Data["Feedback Category"]);

        // The following line demonstrates how we return all categories from the query, already sorted
        return Sxc.Serializer.Prepare(App.Query["All Categories"]["Default"]);

        // The following line demonstrates how we would return the entire query (if it had multiple streams)
        // return Sxc.Serializer.Prepare(App.Query["All Categories"]["Default"]);


        // And this demonstrates how we would return a list of assembled objects with attributes as we would want
        // return from c in AsDynamic(App.Query["All Categories"]["Default"]) select new {
        //    Id = c.EntityId,
        //    Title = c.EntityTitle
        // };
    }

	[HttpPost]
    [DnnModuleAuthorize(AccessLevel = SecurityAccessLevel.Anonymous)]
    [ValidateAntiForgeryToken]
    public bool Create(dynamic postFeedback)
    {
        var feedback = new Dictionary<string, object>();
        feedback.Add("Subject", postFeedback.Subject.ToString());
        feedback.Add("Message", postFeedback.Message.ToString());
        feedback.Add("Status", "New");

        // to define relationships, always create a list first. Reason is that internally
        // relationships could always be more than one, so it's always a list
        // use the entity-id to establish the relationship
        var categ = new List<int>();
        categ.Add(Convert.ToInt32(postFeedback.CategoryId.ToString()));
        feedback.Add("Category", categ);

        App.Data.Create("Feedback", feedback, "Anonymous"); // full version, with "who did it" for the log entry

        return true;
    }

    [HttpPost]
    [DnnModuleAuthorize(AccessLevel = SecurityAccessLevel.Edit)]
    [ValidateAntiForgeryToken]
    public bool Update(int id, dynamic postFeedback)
    {
        var feedback = new Dictionary<string, object>();
        feedback.Add("Subject", postFeedback.Subject.ToString());
        feedback.Add("Message", postFeedback.Message.ToString());
        // feedback.Add("Status", postFeedback.M);

        // to define relationships, always create a list first. Reason is that internally
        // relationships could always be more than one, so it's always a list
        // use the entity-id to establish the relationship
        
        var categ = new List<int>();
        categ.Add(Convert.ToInt32(postFeedback.Category[0].Id.ToString()));
        feedback.Add("Category", categ);

        App.Data.Update(id, feedback, "Anonymous"); // full version, with "who did it" for the log entry

        return true;
    }

    [HttpGet]
    [DnnModuleAuthorize(AccessLevel = SecurityAccessLevel.Edit)]
    [ValidateAntiForgeryToken]
    public dynamic All()
    {
        return Sxc.Serializer.Prepare(App.Data["Feedback"]);
    }

    
    [HttpGet]
    [DnnModuleAuthorize(AccessLevel = SecurityAccessLevel.Edit)]
    [ValidateAntiForgeryToken]
    public dynamic Get(int id)
    {
        // The quick implementation - simple, using the visual query designer
        return Sxc.Serializer.Prepare(App.Query["Get One Feedback"]);

        // example of a manual implementation
        // var c = AsDynamic(App.Data["Default"].List[id]);
        // 
        // return new {
        //    Id = c.EntityId,
        //    Subject = c.Subject,
        //    Status = c.Status,
        //    CategoryId = c.Category[0].EntityId,
        //    Message = c.Message
        // };
    }
    
    [HttpGet]
    [DnnModuleAuthorize(AccessLevel = SecurityAccessLevel.Edit)]
    [ValidateAntiForgeryToken]
    public void SetComplete(int id)
    {
        var editSet = new Dictionary<string, object>();
        editSet.Add("Status", "Completed");
        App.Data.Update(id, editSet);
    }

    [HttpGet]
    [DnnModuleAuthorize(AccessLevel = SecurityAccessLevel.Edit)]
    [ValidateAntiForgeryToken]
    public void SetNew(int id)
    {
        var editSet = new Dictionary<string, object>();
        editSet.Add("Status", "New");
        App.Data.Update(id, editSet);
    }

    [HttpDelete]
    [DnnModuleAuthorize(AccessLevel = SecurityAccessLevel.Edit)]
    [ValidateAntiForgeryToken]
    public void Delete(int id)
    {
        App.Data.Delete(id);
    }
}